package com.jkcredit.query.record.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jkcredit.query.record.constant.CommonConstant;
import com.jkcredit.query.record.model.CommonResponseObject;
import com.jkcredit.query.record.model.MonthResult;
import com.jkcredit.query.record.service.QueryRecordService;
import com.jkcredit.query.record.util.IndexUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.core.CountRequest;
import org.elasticsearch.client.core.CountResponse;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Service
@Slf4j
public class QueryRecordServiceImpl implements QueryRecordService {
    @Autowired
    private RestHighLevelClient esRestClient;
    private static SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

    @Override
    public CommonResponseObject monthRecords(String plateNumber, String month) {
        String[] indices = {CommonConstant.ES_INDEX + month};
        List<MonthResult> results = find(plateNumber, indices);
        return new CommonResponseObject(CommonConstant.SUCCESS_CODE, JSON.toJSONString(results));
    }

    @Override
    public CommonResponseObject yearRecords(String plateNumber) {
        String[] indices = new String[12];
        LocalDate today = LocalDate.now();
        for (int i = 0; i < 12; i++) {
            LocalDate localDate = today.minusMonths(i);
            String month = localDate.toString().substring(0, 7).replace("-", "");
            indices[i] = CommonConstant.ES_INDEX + month;
        }
        List<MonthResult> results = find(plateNumber, indices);
        return new CommonResponseObject(CommonConstant.SUCCESS_CODE, JSON.toJSONString(results));
    }

    @Override
    public CommonResponseObject countByMonth(String month) {
        String index = CommonConstant.ES_INDEX + month;
        if (!isExistIndex(index)) {
            return new CommonResponseObject(CommonConstant.SUCCESS_CODE, "0");
        }
        //构建判断条件
        BoolQueryBuilder boolBuilder = QueryBuilders.boolQuery();
        searchSourceBuilder.timeout(new TimeValue(CommonConstant.OUT_TIME, TimeUnit.MILLISECONDS));
        searchSourceBuilder.query(boolBuilder);
        CountRequest countRequest = new CountRequest(index);
        CountResponse countResponse;
        long count = 0L;
        try {
            countResponse = esRestClient.count(countRequest, RequestOptions.DEFAULT);
            count = countResponse != null ? countResponse.getCount() : 0;
        } catch (Exception e) {
            log.error("queryRecordError:", e);
        }
        return new CommonResponseObject(CommonConstant.SUCCESS_CODE, String.valueOf(count));
    }

    @Override
    public CommonResponseObject multipleMonthRecords(String plateNumber, String startMonth, String endMonth) {
        List<String> monthList = getMonthBetween(startMonth, endMonth);
        if (monthList.size() > 12 || monthList.size() == 0) {
            return new CommonResponseObject(CommonConstant.MONTH_ERROR_CODE, CommonConstant.MONTH_ERROR);
        }
        String[] indices = new String[monthList.size()];
        for (int i = 0; i < monthList.size(); i++) {
            indices[i] = CommonConstant.ES_INDEX + monthList.get(i);
        }
        List<MonthResult> results = find(plateNumber, indices);
        return new CommonResponseObject(CommonConstant.SUCCESS_CODE, JSON.toJSONString(results));
    }

    private List<MonthResult> find(String plateNumber, String[] indices) {
        List<String> existIndices = new ArrayList<>();
        for (String index : indices) {
            if (isExistIndex(index)) {
                existIndices.add(index);
            }
        }
        //设置es的索引index和类型他type
        SearchRequest searchRequest = new SearchRequest();
        searchRequest.indices(existIndices.toArray(new String[0]));
        //bool工厂，相当于sql中where条件，where之后的条件都写在这里
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();
        queryBuilder.must(QueryBuilders.matchQuery("vehicleid.keyword", plateNumber));
        //将where条件放入工厂
        searchSourceBuilder.timeout(new TimeValue(CommonConstant.OUT_TIME, TimeUnit.MILLISECONDS));
        searchSourceBuilder.query(queryBuilder);
        searchSourceBuilder.size(CommonConstant.MAX_RESULT_SIZE);
        searchRequest.source(searchSourceBuilder);
        SearchResponse searchResponse = null;
        try {
            //执行请求
            searchResponse = esRestClient.search(searchRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            log.error("queryRecordError:", e);
        }
        //返回查询到的具体数据
        SearchHits searchHits = searchResponse.getHits();
        List<MonthResult> results = new ArrayList<>();
        for (SearchHit hit : searchHits.getHits()) {
            MonthResult monthResult = JSON.toJavaObject(JSON.parseObject(hit.getSourceAsString()), MonthResult.class);
            results.add(monthResult);
        }
        JSONObject logObject = new JSONObject();
        logObject.put("plateNumber", plateNumber);
        logObject.put("indices", indices);
        logObject.put("result", results);
        log.info("queryRecordSuccess:{}", logObject.toJSONString());
        return results;
    }

    private boolean isExistIndex(String index) {
        boolean isExistIndex = false;
        String indexInMap = IndexUtil.INSTANCE.getByIndex(index);
        if (StringUtils.isBlank(indexInMap)) {
            GetIndexRequest getIndexRequest = new GetIndexRequest(index);
            try {
                isExistIndex = esRestClient.indices().exists(getIndexRequest, RequestOptions.DEFAULT);
            } catch (Exception e) {
                e.printStackTrace();
            }
            if (isExistIndex) {
                IndexUtil.INSTANCE.put(index, String.valueOf(System.currentTimeMillis()));
            }
        } else {
            isExistIndex = true;
        }

        return isExistIndex;
    }

    private List<String> getMonthBetween(String minDate, String maxDate) {
        ArrayList<String> result = new ArrayList<String>();
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");//格式化为年月

            Calendar min = Calendar.getInstance();
            Calendar max = Calendar.getInstance();
            min.setTime(sdf.parse(minDate));
            min.set(min.get(Calendar.YEAR), min.get(Calendar.MONTH), 1);

            max.setTime(sdf.parse(maxDate));
            max.set(max.get(Calendar.YEAR), max.get(Calendar.MONTH), 2);

            Calendar curr = min;
            while (curr.before(max)) {
                result.add(sdf.format(curr.getTime()));
                curr.add(Calendar.MONTH, 1);
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return result;
    }
}
